@@ -20,20 +20,18 @@ if (caller eq "CGI::Apache") {
 
 {
     no strict;
-    $VERSION = "1.26";
+    $VERSION = "1.27";
     __PACKAGE__->mod_perl::boot($VERSION);
 }
 
-if($ENV{MOD_PERL} && perl_hook("Sections")) {
+BEGIN {
     *Apache::ReadConfig:: = \%ApacheReadConfig::;
+}
 
-    if(Apache::Constants::MODULE_MAGIC_NUMBER() >= 19971026) {
-	*Apache::httpd_conf = sub {
-	    shift;
-	    push @Apache::ReadConfig::PerlConfig,
-	    map "$_\n", @_;
-	};
-    }
+sub httpd_conf {
+    shift;
+    push @Apache::ReadConfig::PerlConfig,
+      map "$_\n", @_;
 }
 
 sub parse_args {
@@ -51,33 +49,39 @@ sub content {
     return unless $ct =~ m!^application/x-www-form-urlencoded!;
     my $buff;
     $r->read($buff, $r->header_in("Content-length"));
-    parse_args(wantarray, $buff);
+    return $buff unless wantarray;
+    parse_args(1, $buff);
 }
 
 sub args {
     my($r, $val) = @_;
-    parse_args(wantarray, 
-	       $val ? $r->query_string($val) : $r->query_string);
+    my $args = @_ > 1 ? $r->query_string($val) : $r->query_string;
+    return $args unless wantarray;
+    parse_args(1, $args);
 }
 
 *READ = \&read unless defined &READ;
 
 sub read {
-    my($r, $bufsiz) = @_[0,2];
+    my($r, $bufsiz, $offset) = @_[0,2,3];
     my($nrd, $buf, $total);
     $nrd = $total = 0;
     $buf = "";
-    $_[1] ||= "";
-    #$_[1] = " " x $bufsiz unless defined $_[1]; #XXX?
+    $_[1] = "" unless $offset;
 
-    $r->hard_timeout("Apache->read");
+    $r->soft_timeout("Apache->read");
 
     while($bufsiz) {
 	$nrd = $r->read_client_block($buf, $bufsiz) || 0;
 	if(defined $nrd and $nrd > 0) {
 	    $bufsiz -= $nrd;
-	    $_[1] .= $buf;
- 	    #substr($_[1], $total, $nrd) = $buf;
+            if ($offset) {
+                substr($_[1], $offset) .= $buf;
+                #$_[1] .= $buf;
+            }
+            else {
+                $_[1] .= $buf;
+            }
 	    $total += $nrd;
 	    next if $bufsiz;
 	    last;
@@ -115,7 +119,7 @@ sub new_read {
 	return 0;
     }
 
-    $r->hard_timeout("Apache->read");
+    $r->soft_timeout("Apache->read");
     
     while($bufsiz) {
 	$nrd = $r->get_client_block($buf, $bufsiz) || 0;
@@ -154,12 +158,6 @@ sub PRINTF {
 }
 *printf = \&PRINTF;
 
-sub WRITE {
-    my($r, $buff, $length, $offset) = @_;
-    my $send = substr($buff, $offset, $length);
-    $r->print($send);
-}
-
 sub send_cgi_header {
     my($r, $headers) = @_;
     my $dlm = "\015?\012"; #a bit borrowed from LWP::UserAgent
@@ -182,7 +180,10 @@ sub send_cgi_header {
 	    else {
 		$not_sent = 1;
 	    }
-	    $r->send_http_header if $not_sent;
+            if ($not_sent) {
+                $r->send_http_header;
+                $r->sent_header(-2);
+            }
 	    $r->print($headers); #send rest of buffer, without stripping newlines!!!
 	    last;
 	}
@@ -421,13 +422,16 @@ returned.  When called in a list context, a list of parsed I<key> =>
 I<value> pairs are returned.  *NOTE*: you can only ask for this once,
 as the entire body is read from the client.
 
-=item $r->read($buf, $bytes_to_read)
+=item $r->read($buf, $bytes_to_read, [$offset])
 
 This method is used to read data from the client, 
 looping until it gets all of C<$bytes_to_read> or a timeout happens.
 
+An offset may be specified to place the read data at some other place
+than the beginning of the string.
+
 In addition, this method sets a timeout before reading with
-C<$r-E<gt>hard_timeout>.
+C<$r-E<gt>soft_timeout>.
 
 =item $r->get_remote_host
 
@@ -579,6 +583,29 @@ Will return a I<HASH> reference blessed into the
 I<Apache::Table> class when called in a scalar context with no
 "key" argument. See I<Apache::Table>.
 
+
+=item $r->dir_config->get( $key )
+
+Returns the value of a per-directory array variable specified by the
+C<PerlAddVar> directive.
+
+   # <Location /foo/bar>
+   # PerlAddVar  Key  Value1
+   # PerlAddVar  Key  Value2
+   # </Location>
+
+   my @val = $r->dir_config->get('Key');
+
+Alternatively in your code you can extend the setting with:
+
+  $r->dir_config->add(Key => 'Value3');
+
+Keys are case-insensitive.
+
+Will return a I<HASH> reference blessed into the
+I<Apache::Table> class when called in a scalar context with no
+"key" argument. See I<Apache::Table>.
+
 =item $r->requires
 
 Returns an array reference of hash references, containing information
@@ -687,11 +714,12 @@ This is the value of the Group directive.
 
 =item $s->loglevel
 
-Returns the value of the current LogLevel. This method is added by
+Get or set the value of the current LogLevel. This method is added by
 the Apache::Log module, which needs to be pulled in.
 
     use Apache::Log;
     print "LogLevel = ", $s->loglevel;
+    $s->loglevel(Apache::Log::DEBUG);
 
 If using Perl 5.005+, the following constants are defined (but not
 exported):
@@ -747,9 +775,11 @@ content to the client.
 
 =over 4
 
-=item $r->send_http_header
+=item $r->send_http_header( [$content_type] )
 
 Send the response line and all headers to the client.
+Takes an optional parameter indicating the content-type of the
+response, i.e. 'text/html'.
 
 This method will create headers from the $r->content_xxx() and
 $r->no_cache() attributes (described below) and then append the
@@ -911,7 +941,21 @@ and the client should be told not to cache it.
 =item $r->print( @list )
 
 This method sends data to the client with C<$r-E<gt>write_client>, but first
-sets a timeout before sending with C<$r-E<gt>hard_timeout>.
+sets a timeout before sending with C<$r-E<gt>soft_timeout>. This method is
+called instead of CORE::print when you use print() in your mod_perl programs.
+
+This method treats scalar references specially. If an item in @list is a
+scalar reference, it will be dereferenced before printing. This is a
+performance optimization which prevents unneeded copying of large strings,
+and it is subtly different from Perl's standard print() behavior.
+
+Example:
+
+   $foo = \"bar"; print($foo);
+
+The result is "bar", not the "SCALAR(0xDEADBEEF)" you might have expected. If
+you really want the reference to be printed out, force it into a scalar
+context by using C<print(scalar($foo))>.
 
 =item $r->send_fd( $filehandle )
 
@@ -1025,6 +1069,13 @@ destroyed.
        warn "registered cleanup called for ", $r->uri, "\n";
    });
 
+Cleanup functions registered in the parent process (before forking)
+will run once when the server is shut down:
+
+   #PerlRequire startup.pl
+   warn "parent pid is $$\n";
+   Apache->server->register_cleanup(sub { warn "server cleanup in $$\n"});
+
 The I<post_connection> method is simply an alias for I<register_cleanup>, 
 as this method may be used to run code after the client connection is closed,
 which may not be a I<cleanup>.
@@ -1104,6 +1155,20 @@ Returns true if the specified callback hook is enabled:
 
 =back
 
+=head1 GLOBAL VARIABLES
+
+=over 4
+
+=item $Apache::Server::Starting
+
+Set to true when the server is starting.
+
+=item $Apache::Server::ReStarting
+
+Set to true when the server is starting.
+
+=back
+
 =head1 SEE ALSO
 
 perl(1),
