@@ -16,7 +16,7 @@ else {
 
 my $is_xs = ($r->uri =~ /_xs/);
 
-my $tests = 65;
+my $tests = 71;
 my $is_win32 = WIN32;
 $tests += 2 unless $is_win32;
 my $test_get_set = Apache->can('set_handlers') && ($tests += 4);
@@ -27,6 +27,7 @@ my $i;
 
 $r->content_type("text/plain");
 $r->content_languages([qw(en)]);
+$r->no_cache(1);
 $r->send_http_header;
 
 $r->print("1..$tests\n");
@@ -69,6 +70,17 @@ unless ($is_win32) {
   test ++$i, (-s $r->finfo) == $ft_def;
 }
 
+my $the_request = $r->the_request;
+$r->the_request(join ' ', map { $r->$_() } qw(method uri protocol));
+test ++$i, $the_request eq $r->the_request;
+printf "old=$the_request, new=%s\n", $r->the_request;
+
+my $doc_root = $r->document_root;
+$r->document_root('/tmp');
+test ++$i, $r->document_root eq '/tmp';
+$r->document_root($doc_root);
+test ++$i, $r->document_root eq $doc_root;
+
 my $loc = $r->location;
 print "<Location $loc>\n";
 test ++$i, $loc and $r->uri =~ m:^$loc:;
@@ -165,6 +177,14 @@ test ++$i, $s;
 test ++$i, $s->server_admin;
 test ++$i, $s->server_hostname;
 test ++$i, $s->port;
+my $port = $s->port;
+for (32768, 65535) {
+    $s->port($_);
+    test ++$i, $s->port; # == $_;
+}
+$s->port($port);
+test ++$i, $s->port == $port;
+
 test ++$i, $s->timeout;
 
 for (my $srv = $r->server; $srv; $srv = $srv->next) {
